//======================================================= file = common.h =====
//=  Include file for VOQ switch simulation model                             =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2)  Basic time unit is the millisecond                           =
//=---------------------------------------------------------------------------=
//=  History:  KJC (01/27/02) - Genesis                                       =
//=============================================================================
#pragma warn -pro                    // Disable function prototype warning
#pragma warn -stu                    // Disable undefined structure warning
#pragma warn -rch                    // Disable unreachable code warning
#pragma warn -ccc                    // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                    // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION "3.21 <01/27/02>"    // Version number
#define OUT_FILE     "result.txt"    // Name of output file

#define FALSE                  0     // FALSE = 0
#define TRUE                   1     // TRUE = 1

#define MEDIA_RATE        10.0e6     // Media rate in bits per millisec
#define N                      2     // Number of input ports in the switch
#define GEN_FLAG               1     // 1 = IEEE disk, 2 = USF dist
#define SCHED_FLAG             2     // 1 = RR, 2 = OPF
#define SPEED_UP             1.0     // Internal transfer speed-up

//----- Overall external variables --------------------------------------------
EXTERN double    Sim_time;           // Simuation time
EXTERN int       Pkt_len[N][N];      // Current packet length
EXTERN double    Time_stamp[N][N];   // Time stamps for arriving cells

//----- External variables for generate1() ------------------------------------
EXTERN double    Lambda1;            // Arrival rate for generate1 in pkts/ms
EXTERN double    Lambda2;            // Arrival rate for generate2 in pkts/ms
EXTERN int       Level[N][N];        // Level of VOQ[i][j] in bytes
EXTERN int       Thresh;             // Threshold for VOQs in bytes
EXTERN int       Burst_in;           // Burst size for VOQs (1 to disable)
EXTERN int       Burst_out;          // Burst size for CPs (1 to disable)
EXTERN int       Count_in[N];        // Burst counter for inport
EXTERN int       Count_out[N][N];    // Burst counter for outport CP buffers

//----- CSIM types ------------------------------------------------------------
EXTERN FACILITY Out_link[N];         // Facility for output port link
EXTERN EVENT    VOQ_first[N][N];     // Input port VOQ HOL for packets
EXTERN EVENT    VOQ_event[N][N];     // Input port VOQ buffers for packets
EXTERN EVENT    CP_event[N][N];      // Crossbar crosspoint queues for packets
EXTERN EVENT    State_change[N];     // State change in an in_id buffer
EXTERN EVENT    Xfer_voq[N];         // VOQ-to-CP buffer xfer done for in_id
EXTERN EVENT    Xfer_cp[N];          // VOQ-to-CP buffer xfer done for out_id
EXTERN EVENT    Xmit_done[N];        // CP-to-link xmit done event for out_id
EXTERN TABLE    Delay_table[N];      // Delay table for packet delay
EXTERN TABLE    Bytes_table[N];      // Bytes transfered table

//----- Function prototypes ---------------------------------------------------
// Traffic generators
EXTERN void generate1(int in_id);
EXTERN void generate2(int in_id);
EXTERN void (* gen[N])(int in_id);

// Input ports with VOQ (VOQ arbiter in each input port)
EXTERN void inport(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void (* in[N])(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void voq_arbit_rr(int in_id);
EXTERN void voq_arbit_opf(int in_id);
EXTERN void (* voq_arb[N])(int in_id);

// Output ports (CP arbiter in each output port)
EXTERN void outport(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void (* out[N])(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void cp_arbit(int out_id);
EXTERN void (* cp_arb[N])(int out_id);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
